//+------------------------------------------------------------------+
//|                                     SignalCustomPatterns_CCI.mqh |
//|                      Copyright  2011, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//|                                              Revision 2011.03.25 |
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//| include files                                                    |
//+------------------------------------------------------------------+
#include <Expert\ExpertSignal.mqh>
#include <Expert\Signal\CustomPatterns.mqh>
// wizard description start
//+------------------------------------------------------------------+
//| Description of the class                                         |
//| Title=Signals based on Custom Candle Patterns confirmed by CCI   |
//| Type=Signal                                                      |
//| Name=SignalCustomPatterns_CCI                                    |
//| Class=SignalCustomPatterns_CCI                                   |
//| Page=                                                            |
//| Parameter=PeriodCCI,int,39                                       |
//| Parameter=LongPattern,int,2                                      |
//| Parameter=ShortPattern,int,1                                     |
//| Parameter=AppliedCCI,ENUM_APPLIED_PRICE,PRICE_CLOSE              |
//| Parameter=MAPeriod,int,5                                         |
//| Parameter=TimeFrame,ENUM_TIMEFRAMES,PERIOD_M1                    |
//+------------------------------------------------------------------+
// wizard description end
//+------------------------------------------------------------------+
//| SignalCustomPatterns_CCI Class.                                  |
//| Purpose: Trading signals class, based on                         |
//| the "Morning Star/Evening Star"                                  |
//| Japanese Candlestick Patterns with confirmation by CCI indicator |
//| Derived from CCandlePattern class.                               |
//+------------------------------------------------------------------+
class SignalCustomPatterns_CCI : public CCustomPattern
  {
protected:
   CiCCI              m_CCI;
   CiMA               m_app_price;
   //--- input parameters
   int                m_periodCCI;
   ENUM_LONG_CANDLE_PATTERNS m_long_pattern;
   ENUM_SHORT_CANDLE_PATTERNS m_short_pattern;
   ENUM_APPLIED_PRICE m_appliedCCI;

public:
   //--- class constructor
                      SignalCustomPatterns_CCI();
   //--- input parameters initialization methods
   void               PeriodCCI(int period)                  { m_periodCCI=period;      }
   void               LongPattern(ENUM_LONG_CANDLE_PATTERNS long_pattern) { m_long_pattern=long_pattern;    }
   void               ShortPattern(ENUM_SHORT_CANDLE_PATTERNS short_pattern) { m_short_pattern=short_pattern;    }
   void               AppliedCCI(ENUM_APPLIED_PRICE applied) { m_appliedCCI=applied;    }
   //--- initialization of indicators
   virtual bool       ValidationSettings();
   virtual bool       InitIndicators(CIndicators *indicators); 
   //--- 
   virtual bool      CheckOpenLong(double& price,double& sl,double& tp,datetime& expiration);
   virtual bool      CheckOpenShort(double& price,double& sl,double& tp,datetime& expiration);
   virtual bool       CheckCloseLong(double &price);
   virtual bool       CheckCloseShort(double &price);

protected:
   //--- CCI initialization method
   bool               InitCCI(CIndicators *indicators);
   bool               InitApplied(CIndicators *indicators);
   //--- CCI line
   double             CCI(int ind) const                     { return(m_CCI.Main(ind)); }
  };
//+------------------------------------------------------------------+
//| SignalCustomPatterns_CCI class constructor.                                    |
//| INPUT:  no.                                                      |
//| OUTPUT: no.                                                      |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
void SignalCustomPatterns_CCI::SignalCustomPatterns_CCI()
  {
//--- set default inputs
   m_periodCCI =35;
   m_long_pattern = CANDLE_PATTERN_THREE_WHITE_SOLDIERS;
   m_short_pattern = CANDLE_PATTERN_THREE_BLACK_CROWS;
   m_appliedCCI=PRICE_CLOSE;
  }
//+------------------------------------------------------------------+
//| Validation settings protected data.                              |
//| INPUT:  no.                                                      |
//| OUTPUT: true-if settings are correct, false otherwise.           |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool SignalCustomPatterns_CCI::ValidationSettings()
  {
//--- call of parent method
   if(!CCustomPattern::ValidationSettings()) return(false);
//--- initial data checks
   if(m_periodCCI<=0)
     {
      printf(__FUNCTION__+": period CCI must be greater than 0");
      return(false);
     }
//--- ok
   return(true);
  }
//+------------------------------------------------------------------+
//| Create indicators.                                               |
//| INPUT:  indicators -pointer of indicator collection.             |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool SignalCustomPatterns_CCI::InitIndicators(CIndicators *indicators)
  {
//--- check
   if(indicators==NULL)                            return(false);
//--- call of parent method
   if(!CCustomPattern::InitIndicators(indicators)) return(false);  
//--- create and initialize CCI indicator
   if(!InitCCI(indicators))                        return(false);
//--- create and initialize Price series
   if(!InitApplied(indicators))                    return(false);
//--- ok
   return(true);
  }
//+------------------------------------------------------------------+
//| Initialization of CCI indicator.                                 |
//| INPUT:  indicators -pointer of indicator collection.             |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool SignalCustomPatterns_CCI::InitCCI(CIndicators *indicators)
  {
//--- add CCI indicator to collection
   if(!indicators.Add(GetPointer(m_CCI)))
     {
      printf(__FUNCTION__+": error adding object");
      return(false);
     }
//--- initialize CCI indicator
   if(!m_CCI.Create(m_symbol.Name(),m_period,m_periodCCI,m_appliedCCI))
     {
      printf(__FUNCTION__+": error initializing object");
      return(false);
     }
   m_CCI.BufferResize(50);
//--- ok
   return(true);
  }
//+------------------------------------------------------------------+
//| Create Applied Price indicator.                                  |
//| INPUT:  indicators -pointer of indicator collection.             |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool SignalCustomPatterns_CCI::InitApplied(CIndicators *indicators)
  {
//--- add Price indicator to collection
   if(!indicators.Add(GetPointer(m_app_price)))
     {
      printf(__FUNCTION__+": error adding object");
      return(false);
     }
//--- initialize Price indicator
   if(!m_app_price.Create(m_symbol.Name(),m_period,1,0,MODE_SMA,m_appliedCCI))
     {
      printf(__FUNCTION__+": error initializing object");
      return(false);
     }
   m_app_price.BufferResize(50);
   m_app_price.FullRelease(true);
//--- ok
   return(true);
  }
//+------------------------------------------------------------------+
//| Check conditions for long position open.                         |
//| INPUT:  price      - refernce for price,                         |
//|         sl         - refernce for stop loss,                     |
//|         tp         - refernce for take profit,                   |
//|         expiration - refernce for expiration.                    |
//| OUTPUT: true-if condition performed, false otherwise.            |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool SignalCustomPatterns_CCI::CheckOpenLong(double &price,double &sl,double &tp,datetime &expiration)
  {
//--- check formation of Morning Star pattern
  if (!CheckLongCandlestickPattern(m_long_pattern)) return(false);
//--- check CCI
  if (!(CCI(1)<-50))                                                  return(false);
//--- ok, use market orders
   price=0.0;
   sl   =0.0;
   tp   =0.0;
//--- set signal to open long position
   return(true);
  }
//+------------------------------------------------------------------+
//| Check conditions for long position close.                        |
//| INPUT:  price - refernce for price.                              |
//| OUTPUT: true-if condition performed, false otherwise.            |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool SignalCustomPatterns_CCI::CheckCloseLong(double &price)
  {
//--- check conditions of long position closing
   if(!(((CCI(1)<80) && (CCI(2)>80)) ||
        ((CCI(1)<-80) && (CCI(2)>-80)))) return(false);
//--- ok, use market orders
   price=0.0;
//--- set signal to close long position
   return(true);
  }
//+------------------------------------------------------------------+
//| Check conditions for short position open.                        |
//| INPUT:  price      - refernce for price,                         |
//|         sl         - refernce for stop loss,                     |
//|         tp         - refernce for take profit,                   |
//|         expiration - refernce for expiration.                    |
//| OUTPUT: true-if condition performed, false otherwise.            |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool SignalCustomPatterns_CCI::CheckOpenShort(double &price,double &sl,double &tp,datetime &expiration)
  {
//--- check formation of Evening Star pattern
  if (!CheckShortCandlestickPattern(m_short_pattern)) return(false);
//--- check CCI
  if (!(CCI(1)>50))                                          return(false);
//--- ok, use market orders
   price=0.0;
   sl   =0.0;
   tp   =0.0;
//--- set signal to open short position
   return(true);
  }
//+------------------------------------------------------------------+
//| Check conditions for short position close.                       |
//| INPUT:  price - refernce for price.                              |
//| OUTPUT: true-if condition performed, false otherwise.            |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool SignalCustomPatterns_CCI::CheckCloseShort(double &price)
  {
//--- check conditions of short position closing
   if(!((CCI(1)>-80) && (CCI(2)<-80)) ||
       ((CCI(1)<80) && (CCI(2)>80))) return(false);
//--- ok, use market orders
   price=0.0;
//--- set signal to close short position
   return(true);
  }
//+------------------------------------------------------------------+
